#!/bin/sh
#
# Licensed Material of Hitachi, Ltd.
# Reproduction, use, modification or disclosure otherwise than
# permitted in the License Agreement is strictly prohibited.
#
# All Rights Reserved. Copyright (C) 2012, Hitachi, Ltd.
#

# process check and execute shell program

WATCHINTERVAL=2             # seconds
EXECRETRYCNT=2              # counts
STOPCLEARSEC=86400          # seconds
EXECRETRYSEC=0              # seconds


PATH=/bin:/usr/bin:$PATH
umask 022

SHDIRNAME=$(cd $(dirname $0); pwd)
SHEXENAME=`basename $0`
if [ "`basename ${SHDIRNAME}`" = "bin" ]
then
    LOGDIR="`dirname ${SHDIRNAME}`/log"
else
    LOGDIR="${SHDIRNAME}"
fi

# Check options.
if [ $# -ne 1 ]
then
    # NOT SPECIFIED TARGET PROCESS
    exit 1
fi
if [ "`which $1 2> /dev/null`" = "" ]
then
    # NOT FOUND TARGET PROCESS
    exit 5
fi
# TARGET=$(cd $(dirname $1); pwd)/`basename $1`
TARGET=$1

PIDFILE="${LOGDIR}/${SHEXENAME}.PID"

# Check previous running process
PSOUT=`ps -eo pid,cmd`
MYPSN=`echo "${PSOUT}" | grep "^ *$$ "`
MYCMD=`echo "${MYPSN}" | sed -e "s/^ *$$ //"`
CMDLS=`echo "${PSOUT}" | grep "^ *[0-9][0-9]* ${MYCMD}\$"`
PSOUT=
for PID in `echo "${CMDLS}" | sed -e "s/^ *\([0-9]*\) .*/\1/" 2> /dev/null`
do
    if [ "$$" != "${PID}" ]
    then
        # ALREADY RUNNING
        exit 1
    fi
done


# Make PID file
PID=$$
echo ${PID} > "${PIDFILE}" 2> /dev/null
echo -17 > /proc/${PID}/oom_adj 2> /dev/null
TARGETPID=0

# START THIS SHELL PROCESS


# Terminate handler
function handler
{
    # search target process (get pid)
    for ((LOOP = 0; LOOP <= 3; LOOP++))
    do
        ISALIVE=`pgrep -P 1,${PID} -xf ${TARGET}`
        if [ -n "${ISALIVE}" ]
        then
            break
        fi
        /bin/sleep 0.2
    done

    # stop target process
    if [ -n "${ISALIVE}" ]
    then
        # STOP TARGET PROCESS
        kill -TERM ${ISALIVE}
    fi

    rm -f "${PIDFILE}"
    # STOP THIS SHELL PROCESS
    exit 0
}


# Set signal trap
trap handler INT TERM HUP


# Check process
RETRYCNT=`expr ${EXECRETRYCNT} + 1`
STOPTIME=0
while [ 1 ]
do
    # search target process (get pid)
    for ((LOOP = 0; LOOP <= 3; LOOP++))
    do
        ISALIVE=`pgrep -P 1,${PID} -xf ${TARGET}`
        if [ -n "${ISALIVE}" ]
        then
            break
        fi
        /bin/sleep 0.2
    done

    # start target process
    if [ -n "${ISALIVE}" ]
    then
        if [ "${ISALIVE}" -ne "${TARGETPID}" ]
        then
            echo -17 > /proc/${ISALIVE}/oom_adj 2> /dev/null
            TARGETPID=${ISALIVE}
        fi
    elif [ ${RETRYCNT} -gt 0 ]
    then
        # NOT FOUND TARGET PROCESS
        if [ ${RETRYCNT} -le ${EXECRETRYCNT} -a ${STOPTIME} -eq 0 ]
        then
            STOPTIME=`date +%s`
        fi
        RETRYCNT=`expr ${RETRYCNT} - 1`

        if [ ${EXECRETRYSEC} -gt 0 ]
        then
            sleep ${EXECRETRYSEC}
        fi
        # EXECUTE TARGET PROCESS
        ${TARGET} > /dev/null 2>&1 &
        TARGETPID=0
    fi

    sleep ${WATCHINTERVAL}
    NOWTIME=`date +%s`
    if [ ${STOPTIME} -ne 0 -a `expr ${STOPCLEARSEC} + ${STOPTIME}` -lt ${NOWTIME} ]
    then
        # CLEAR TARGET PROCESS STOP TIME
        RETRYCNT=${EXECRETRYCNT}
        STOPTIME=0
    fi
done
